@@ -91,33 +91,25 @@ NumericMatrix xydist2cpp (
 //--------------------------------------------------------------------------
 
 // customised dpois 
-double gpois (int count, double lambda, int uselog)
+double gpois (int count, double lambda)
 {
     double x;
     if ((count < 0) || (count>0 && lambda <= 0)) {
-        if (uselog) 
-            return(NAN);
-        else 
-            return(0);
+        return(0);
     }
     else if (count == 0) {
-        if (uselog)
-            return (-lambda);
-        else
-            return (exp(-lambda));
+        return (exp(-lambda));
     }
     else {
-        // return (R::dpois(count, lambda, uselog));
         boost::math::poisson_distribution<> pois(lambda);
         x = boost::math::pdf(pois, count);
-        if (uselog) x = log(x);
         return (x);
     }
 }
 //--------------------------------------------------------------------------
 
 // customised dbinom 
-double gbinom(int count, int size, double p, int uselog)
+double gbinom(int count, int size, double p)
 {
     double x, q;
     int i;
@@ -130,42 +122,13 @@ double gbinom(int count, int size, double p, int uselog)
         for (i=1; i< size; i++) x *= q;
     }
     else {
-        // x = R::dbinom(count, size, p, 0);
         boost::math::binomial_distribution<> bin(size, p);
         x = boost::math::pdf(bin, count);
     }
-    if (uselog) {
-        if (x <= 0) 
-            x = NAN;
-        else
-            x = log(x);
-    }
     return (x);   
 }
 //--------------------------------------------------------------------------
 
-// not used 2021-10-17
-// customised dnbinom parameterised as size, mu 
-// double gnbinom (int count, int size, double mu, int uselog)
-// {
-//     // prob = size / (size + mu) 
-//     // size = fabs(size);   in case negative 'binomN' passed 
-//     size = abs(size);  // in case negative 'binomN' passed 
-//     
-//     if (count == 0) {  // faster - added 2010-10-11
-//         if (uselog) return( log(size/(size+mu)) * log(size) );
-//         else return (pow(size/(size+mu), size));
-//     }
-//     else {
-//         // return (R::dnbinom (count, size, size/(size+mu), uselog));
-//         boost::math::binomial_distribution<> bin(size, size/(size+mu));
-//         double x = boost::math::pdf(bin, count);
-//         if (uselog) x = log(x);
-//         return (x);
-//     }
-// }
-//--------------------------------------------------------------------------
-
 // probability of count with distribution specified by binomN 
 double countp (int count, int binomN, double lambda) {
     // Poisson 
@@ -189,14 +152,12 @@ double countp (int count, int binomN, double lambda) {
     
     // negative binomial 
     else if (binomN < 0) {
-        // return ( gnbinom (count, binomN, lambda, 0) );
         boost::math::negative_binomial_distribution<> nbin(binomN, lambda);
         return (boost::math::pdf(nbin, count));
     }
     
     // binomial 
     else {
-        // return ( gbinom (count, binomN, lambda, 0) ); 
         boost::math::binomial_distribution<> bin(binomN, lambda);
         return (boost::math::pdf(bin, count));
     }
@@ -350,32 +311,6 @@ rpoint getxy(
 }
 //--------------------------------------------------------------------------
 
-rpoint getxycpp(
-        const double l, 
-        const std::vector<double> &cumd, 
-        const RcppParallel::RMatrix<double> &line, 
-        const int kk, 
-        const double offset) {
-    // return the xy coordinates of point l metres along a transect 
-    // offset is the starting position for this transect 
-    int k;
-    double pr, d, d12;
-    rpoint xy;
-    for (k=offset+1; k<(offset+kk); k++) {
-        if (cumd[k]>l) break;
-    }
-    d = l - cumd[k-1];  // distance along leg 
-    d12 = cumd[k] - cumd[k-1];
-    if (d12>0)
-        pr = d / d12;
-    else
-        pr = 0;
-    xy.x = line(k-1,0) + (line(k,0) - line(k-1,0)) * pr;
-    xy.y = line(k-1,1) + (line(k,1) - line(k-1,1)) * pr;
-    return(xy);
-}
-//--------------------------------------------------------------------------
-
 // Calculate the length of intersection of a line segment and a circle
 // Based on C code of Paul Bourke November 1992
 // Line segment is defined from p1 to p2
@@ -703,7 +638,6 @@ double rcount (int binomN, double lambda, const double Tsk) {
             else
                 return (0);
         }
-        
         // binomial 
         else
             return (R::rbinom(binomN, lambda) );
@@ -1300,19 +1234,19 @@ double pski ( int binomN,
             result = exp(-lambda);            // routinely apply Tsk adjustment to cum. hazard 
         }
         else {
-            //result = R::dpois(count, Tski * g * pI, 0); 
+            //result = R::dpois(count, Tski * g * pI); 
             boost::math::poisson_distribution<> pois(lambda);
             result = boost::math::pdf(pois,count);
         }
     }
     else if (binomN == 1) {                          // count detectors : Binomial, size from Tsk
-        result = gbinom (count, round(Tski), g*pI, 0); 
+        result = gbinom (count, round(Tski), g*pI); 
     }
     else if (binomN > 1) {                           // count detectors : Binomial, specified size 
         if (abs(Tski-1) > 1e-10) {                   // effort not unity, adjust g 
             g = 1 - pow(1 - g, Tski);
         }
-        result = gbinom (count, binomN, g*pI, 0);
+        result = gbinom (count, binomN, g*pI);
     }
     else result = NAN; // Rcpp::stop("binomN < -1 not allowed");  // code multi -2 separately
     
